@@ -9,8 +9,6 @@
 
 #include "matrix.h"
 
-#include <assert.h>
-
 #include <random>
 #include <exception>
 #include <stdexcept>
@@ -20,43 +18,12 @@
 
 namespace fasttext {
 
-Matrix::Matrix() {
-  m_ = 0;
-  n_ = 0;
-  data_ = nullptr;
-}
+Matrix::Matrix() : Matrix(0, 0) {}
 
-Matrix::Matrix(int64_t m, int64_t n) {
-  m_ = m;
-  n_ = n;
-  data_ = new real[m * n];
-}
-
-Matrix::Matrix(const Matrix& other) {
-  m_ = other.m_;
-  n_ = other.n_;
-  data_ = new real[m_ * n_];
-  for (int64_t i = 0; i < (m_ * n_); i++) {
-    data_[i] = other.data_[i];
-  }
-}
-
-Matrix& Matrix::operator=(const Matrix& other) {
-  Matrix temp(other);
-  m_ = temp.m_;
-  n_ = temp.n_;
-  std::swap(data_, temp.data_);
-  return *this;
-}
-
-Matrix::~Matrix() {
-  delete[] data_;
-}
+Matrix::Matrix(int64_t m, int64_t n) : data_(m * n), m_(m), n_(n) {}
 
 void Matrix::zero() {
-  for (int64_t i = 0; i < (m_ * n_); i++) {
-      data_[i] = 0.0;
-  }
+  std::fill(data_.begin(), data_.end(), 0.0);
 }
 
 void Matrix::uniform(real a) {
@@ -73,7 +40,7 @@ real Matrix::dotRow(const Vector& vec, int64_t i) const {
   assert(vec.size() == n_);
   real d = 0.0;
   for (int64_t j = 0; j < n_; j++) {
-    d += at(i, j) * vec.data_[j];
+    d += at(i, j) * vec[j];
   }
   if (std::isnan(d)) {
     throw std::runtime_error("Encountered NaN.");
@@ -86,15 +53,17 @@ void Matrix::addRow(const Vector& vec, int64_t i, real a) {
   assert(i < m_);
   assert(vec.size() == n_);
   for (int64_t j = 0; j < n_; j++) {
-    data_[i * n_ + j] += a * vec.data_[j];
+    data_[i * n_ + j] += a * vec[j];
   }
 }
 
 void Matrix::multiplyRow(const Vector& nums, int64_t ib, int64_t ie) {
-  if (ie == -1) {ie = m_;}
+  if (ie == -1) {
+    ie = m_;
+  }
   assert(ie <= nums.size());
   for (auto i = ib; i < ie; i++) {
-    real n = nums[i-ib];
+    real n = nums[i - ib];
     if (n != 0) {
       for (auto j = 0; j < n_; j++) {
         at(i, j) *= n;
@@ -104,10 +73,12 @@ void Matrix::multiplyRow(const Vector& nums, int64_t ib, int64_t ie) {
 }
 
 void Matrix::divideRow(const Vector& denoms, int64_t ib, int64_t ie) {
-  if (ie == -1) {ie = m_;}
+  if (ie == -1) {
+    ie = m_;
+  }
   assert(ie <= denoms.size());
   for (auto i = ib; i < ie; i++) {
-    real n = denoms[i-ib];
+    real n = denoms[i - ib];
     if (n != 0) {
       for (auto j = 0; j < n_; j++) {
         at(i, j) /= n;
@@ -119,8 +90,7 @@ void Matrix::divideRow(const Vector& denoms, int64_t ib, int64_t ie) {
 real Matrix::l2NormRow(int64_t i) const {
   auto norm = 0.0;
   for (auto j = 0; j < n_; j++) {
-    const real v = at(i,j);
-    norm += v * v;
+    norm += at(i, j) * at(i, j);
   }
   if (std::isnan(norm)) {
     throw std::runtime_error("Encountered NaN.");
@@ -130,23 +100,35 @@ real Matrix::l2NormRow(int64_t i) const {
 
 void Matrix::l2NormRow(Vector& norms) const {
   assert(norms.size() == m_);
-    for (auto i = 0; i < m_; i++) {
-      norms[i] = l2NormRow(i);
-    }
+  for (auto i = 0; i < m_; i++) {
+    norms[i] = l2NormRow(i);
+  }
 }
 
 void Matrix::save(std::ostream& out) {
-  out.write((char*) &m_, sizeof(int64_t));
-  out.write((char*) &n_, sizeof(int64_t));
-  out.write((char*) data_, m_ * n_ * sizeof(real));
+  out.write((char*)&m_, sizeof(int64_t));
+  out.write((char*)&n_, sizeof(int64_t));
+  out.write((char*)data_.data(), m_ * n_ * sizeof(real));
 }
 
 void Matrix::load(std::istream& in) {
-  in.read((char*) &m_, sizeof(int64_t));
-  in.read((char*) &n_, sizeof(int64_t));
-  delete[] data_;
-  data_ = new real[m_ * n_];
-  in.read((char*) data_, m_ * n_ * sizeof(real));
+  in.read((char*)&m_, sizeof(int64_t));
+  in.read((char*)&n_, sizeof(int64_t));
+  data_ = std::vector<real>(m_ * n_);
+  in.read((char*)data_.data(), m_ * n_ * sizeof(real));
 }
 
+void Matrix::dump(std::ostream& out) const {
+  out << m_ << " " << n_ << std::endl;
+  for (int64_t i = 0; i < m_; i++) {
+    for (int64_t j = 0; j < n_; j++) {
+      if (j > 0) {
+        out << " ";
+      }
+      out << at(i, j);
+    }
+    out << std::endl;
+  }
+};
+
 }
