@@ -30,8 +30,8 @@ class QMatrix {
     std::unique_ptr<ProductQuantizer> pq_;
     std::unique_ptr<ProductQuantizer> npq_;
 
-    uint8_t* codes_;
-    uint8_t* norm_codes_;
+    std::vector<uint8_t> codes_;
+    std::vector<uint8_t> norm_codes_;
 
     bool qnorm_;
 
@@ -44,7 +44,6 @@ class QMatrix {
 
     QMatrix();
     QMatrix(const Matrix&, int32_t, bool);
-    ~QMatrix();
 
     int64_t getM() const;
     int64_t getN() const;
